@@ -5,7 +5,7 @@ use warnings;
 package Sub::HandlesVia::Handler;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.002';
+our $VERSION   = '0.003';
 
 use Class::Tiny (
 	qw(
@@ -334,20 +334,20 @@ sub install_method {
 	
 	if ( eval { require Sub::Util }) {
 		$coderef = Sub::Util::set_subname("$target\::$name", $coderef);
-		no strict 'refs';
-		*{"$target\::$name"} = $coderef;
 	}
 	elsif ( eval { require Sub::Name }) {
 		$coderef = Sub::Name::subname("$target\::$name", $coderef);
-		no strict 'refs';
-		*{"$target\::$name"} = $coderef;
+	}
+	
+	if ($callbacks{install_method}) {
+		$callbacks{install_method}->($name, $coderef);
+	}
+	elsif ($callbacks{install_method_fq}) {
+		$callbacks{install_method}->("$target\::$name", $coderef);
 	}
 	else {
-		eval qq{
-			package $target;
-			sub $name { goto \$coderef };
-			1;
-		} or die($@);
+		no strict 'refs';
+		*{"$target\::$name"} = $coderef;
 	}
 }
 
@@ -381,7 +381,7 @@ sub _generate_handler {
 package Sub::HandlesVia::Handler::Traditional;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.002';
+our $VERSION   = '0.003';
 
 BEGIN { our @ISA = 'Sub::HandlesVia::Handler' };
 
@@ -422,7 +422,7 @@ sub _coderef {
 package Sub::HandlesVia::Handler::CodeRef;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.002';
+our $VERSION   = '0.003';
 
 BEGIN { our @ISA = 'Sub::HandlesVia::Handler' };
 
