@@ -5,7 +5,7 @@ use warnings;
 package Sub::HandlesVia::Toolkit;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.008_003';
+our $VERSION   = '0.003';
 
 use Type::Params qw(compile_named_oo);
 use Types::Standard qw( ArrayRef HashRef Str Num Int CodeRef Bool );
@@ -49,7 +49,6 @@ my %native = qw(
 	Counter         1
 	Hash            1
 	Number          1
-	Scalar          1
 	String          1
 );
 
@@ -91,8 +90,8 @@ sub clean_spec {
 	my $joined      = join('|', @handles_via);
 
 	if ($default_type{$joined} and not exists $spec->{isa}) {
-		$spec->{isa}    = $default_type{$joined};
-		$spec->{coerce} = 1 if $default_type{$joined}->has_coercion;
+		$spec->{isa}    =    $default_type{$joined};
+		$spec->{coerce} = !! $default_type{$joined}->has_coercion;
 	}
 	
 	$spec->{handles} = { map +($_ => $_), @{ $spec->{handles} } }
